/*
 * Universitat Oberta de Catalunya (UOC)
 * Projecte de Final de Carrera :
 * Disseny i implementació d'un Framework de presentació
 * per aplicacions J2EE
 * 
 * Alumne : Miquel Vaquer Menorca
 */

package fw.config;

import fw.action.Action;
import fw.action.ActionForm;
import fw.exceptions.TomirException;
import java.util.*;
import java.io.File;
import java.net.URL;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.Logger;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Mètode que gestiona el mapeig de les accions, actionForms i vistes de la
 * petició. Implementa el patró Singleton, ja que només s'ha de instanciar
 * una vegada i proporcionar un punt d'accés global a la configuració.
 *
 * @author Miquel Vaquer Menorca
 */
public class Configuracio 
{
    // Inicialitzar el logging amb log4j
    private static final Logger log;
    static {
        BasicConfigurator.configure();
        log = Logger.getLogger("Configuracio.class");
    }
    
    // Instanciar la configuració. Es defineix private ja que només
    // s'accedirà desde aquesta classe (patró SINGLETON)
    private static Configuracio configuracio;

    // Hashmap on es guarden les dades de la configuració del Framework,
    // mapa d'Accions i de Formularis
    HashMap<String, Action>     actions;
    HashMap<String, ActionForm> actionForms;
    
    /**
     * Constructor, es declara private per evitar la instanciació
     * desde altres classes
     */
    private Configuracio(){
        this.actions     = new HashMap<String, Action>();
        this.actionForms = new HashMap<String, ActionForm>();
    }

    /**
     * Mètode que retorna l'objecte Configuracio, si no existeix
     * es crea un objecte nou. (Patró SINGLETON)
     * @return configuracio
     */
    public static Configuracio getConfiguracio() 
    {
        if (configuracio == null) configuracio = new Configuracio();
        return configuracio;
    }
    
    /**
     * Retorna l'Action del hashmap de Actions 
     * 
     * @param nom_accio
     * @return action
     */
    public Action getAction(String nom_accio) 
    {
        return this.actions.get(nom_accio);
    }

    /**
     * Mètode que instancia una nova accio (Action)
     * Es crea una classe Action de forma dinàmica utilitzant tècniques
       de reflexió. Aquesta tècnica en Java, permet modificar l'estructura
       del codi de manera dinàmica (en temps d'execució)
       * 
     * @param nom_accio
     * @param nom_classe
     * @return 
     */
    private Action newAction(String nom_accio, String nom_classe) 
    {
        String nomClass = nom_classe + "." + nom_accio;
                
        Class classe;
        Action action = null;
        log.info("newAction-->nom_accio= "+nom_accio);
        
        try {
            classe = Class.forName(nomClass);
            action = (Action) classe.newInstance();
        } 
        catch (Exception e) 
        {
            log.error("newAction-->Error in Action " + nom_accio);
            throw new TomirException("Error creant l'acció "+e.getMessage());
        }
        return action;
    }

    /**
     * Mètode que retorna un ActionForm (Formulari)
     * 
     * @param nom_ActionForm
     * @return ActionForm
     */
    public ActionForm getActionForm(String nom_ActionForm) 
    {
        log.info("getActionForm-->actionForm= "+this.actionForms.get(nom_ActionForm));
        return this.actionForms.get(nom_ActionForm);
    }   

    /**
     * Mètode que instancia un nou Formulari(ActionForm)
     * Es crea una classe ActionForm de forma dinàmica utilitzant tècniques
       de reflexió. Aquesta tècnica en Java, permet modificar l'estructura
       del codi de manera dinàmica (en temps d'execució)
     * 
     * @param nom_ActionForm
     * @return actionForm
     */
    private ActionForm newActionForm(String nom_ActionForm, String nom_classe) 
    {
        String nomClass = nom_classe + "." + nom_ActionForm;
        
        Class classe;
        ActionForm actionForm = null;
        log.info("newActionForm-->nom_ActionForm= "+nom_ActionForm);
        
        try 
        {
            classe = Class.forName(nomClass);
            actionForm = (ActionForm) classe.newInstance();
        } 
        catch (Exception e) 
        {
            log.error("newActionForm-->Error in ActionForm " + nom_ActionForm);
            throw new TomirException("Error creant l'ActionForm "+e.getMessage());
            
        }
        return actionForm;
    }
    
    /**
     * Mètode que parsea els valors de Configuracio (Actions, Formularis i Jsp)
     * Utilitzant DOM, llegeix el fitxer de configuració (tomir-config.xml)
     * i va introduint els valors a les taules hash corresponents.
     */
    public void parserTOMIR()
    {
        try 
        {
            log.info("Inici parserTOMIR");
            URL xml = this.getClass().getResource("/tomir-config.xml");
            File tomir = new File(xml.getPath());
            DocumentBuilderFactory dbFactory = DocumentBuilderFactory.newInstance();
            DocumentBuilder dBuilder = dbFactory.newDocumentBuilder();
            Document doc = dBuilder.parse(tomir);
            doc.getDocumentElement().normalize();
            NodeList nodes;

            nodes = doc.getElementsByTagName("accio");
            for (int i = 0; i < nodes.getLength(); i++) 
            {
                Node node = nodes.item(i);
                if (node.getNodeType() == Node.ELEMENT_NODE) {
                    Element element = (Element) node;
                    Action action = newAction(getValue("nom", element), getValue("classe", element));
                    this.actions.put(getValue("nom", element), action);
                    log.info("parserTOMIR-->AfegirAccio= "+action.toString());
                }
            }
            
            nodes = doc.getElementsByTagName("formulari");
            for (int i = 0; i < nodes.getLength(); i++) {
                Node node = nodes.item(i);

                if (node.getNodeType() == Node.ELEMENT_NODE) {
                    Element element = (Element) node;
                    ActionForm actionForm = newActionForm(getValue("nom", element), getValue("classe", element));
                    this.actionForms.put(getValue("action", element), actionForm);
                    log.info("parserTOMIR-->AfegirFormulari= "+actionForm.toString());
                }
            }
        } 
        catch (Exception e) 
        {
            log.error("ParserTOMIR-->Error en el parseig");
            throw new TomirException("Error en el parseig del fields "+e.getMessage());
        }
    }

    private static String getValue(String tag, Element element) {
        NodeList nodes = element.getElementsByTagName(tag).item(0).getChildNodes();
        Node node = (Node) nodes.item(0);
        return node.getNodeValue();
    }       
}